#include "utility_lock.hpp"
#include "item_db.hpp"
#include "utils/godot_macros.hpp"

void UtilityLock::_bind_methods() {
#define CLASSNAME UtilityLock
    GDPROPERTY_HINTED(allowed_items, gd::Variant::ARRAY, gd::PROPERTY_HINT_ARRAY_TYPE, ItemDB::get_array_hint());
    GDSIGNAL("begin_activate", gd::PropertyInfo(gd::Variant::INT, "with_item"), gd::PropertyInfo(gd::Variant::OBJECT, "by_unit", gd::PROPERTY_HINT_NODE_TYPE, "Unit"));
    GDSIGNAL("finish_activate", gd::PropertyInfo(gd::Variant::INT, "with_item"), gd::PropertyInfo(gd::Variant::OBJECT, "by_unit", gd::PROPERTY_HINT_NODE_TYPE, "Unit"));
}

bool UtilityLock::can_use(Item const *with, Unit *by) const {
    return this->allowed_items.has(with);
}

bool UtilityLock::begin_activate(Item const *with, Unit *by) {
    if(this->can_use(with, by)) {
        this->user = by;
        this->emit_signal("begin_activate", with->get_id(), by);
        this->_begin_activate(with, by);
        return true;
    } else return false;
}

bool UtilityLock::finish_activate(Item const *with, Unit *by) {
    if(this->user != nullptr && this->user == by) {
        this->user = nullptr;
        this->emit_signal("finish_activate", with->get_id(), by);
        this->_finish_activate(with, by);
        return true;
    } else return false;
}

void UtilityLock::set_allowed_items(gd::Array array) {
    this->allowed_items.clear();
    for(int i{0}; i < array.size(); ++i) {
        Item const *item{ItemDB::get_item(array[i])};
        if(item == nullptr || item->has_capability(ItemCapability::UtilityEquip))
            this->allowed_items.insert(item);
    }
}

gd::Array UtilityLock::get_allowed_items() const {
    gd::Array array{};
    for(Item const *item : this->allowed_items)
        array.push_back(item == nullptr ? 0 : item->get_id());
    return array;
}
